@@ -285,7 +285,7 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
         Serialize a given graph into GraphML string or return None if graph not found
         :return:
         """
-        inner_query = f'match(n {{GraphID: "{self.graph_id}"}}) -[r]- (m) return n, r, m'
+        inner_query = f'match(n {{GraphID: "{self.graph_id}"}}) optional match(n) -[r]- (m) return n, r, m'
         # run inner query to check the graph has anything in it
         with self.driver.session() as session:
             val = session.run(inner_query)
@@ -421,13 +421,47 @@ class Neo4jPropertyGraph(ABCPropertyGraph):
         :param label:
         :return:
         """
-        raise RuntimeError("Method node implemented")
+        assert node_id is not None
+        assert label is not None
+        query = f"MATCH (n:{label} {{GraphID: $graphId, NodeID: $nodeId}} RETURN collect(n.NodeID) as nodeids"
+        with self.driver.session() as session:
+            val = session.run(query, graphId=self.graph_id, nodeId=node_id).single()
+            if val is None or len(val.data()) == 0 or len(val.data()['nodeids']) == 0:
+                return False
+            return True
 
-    def add_node(self, *, node_id: str, label: str, props: Dict[str, Any]) -> None:
-        raise RuntimeError("Method not implemented")
+    def add_node(self, *, node_id: str, label: str, props: Dict[str, Any] = None) -> None:
+
+        assert node_id is not None
+        assert label is not None
+
+        all_props = f"Class: '{label}', GraphID: '{self.graph_id}', NodeID: '{node_id}', "
+        if props is not None:
+            for k, v in props.items():
+                all_props += f"{k}: '{v}', "
+        all_props = all_props[:-2]
+        labels = f"'GraphNode', '{label}'"
+        query = f"CALL apoc.create.node([ {labels} ], {{ {all_props} }});"
+        with self.driver.session() as session:
+            session.run(query)
 
     def add_link(self, *, node_a: str, rel: str, node_b: str, props: Dict[str, Any] = None) -> None:
-        raise RuntimeError("Method not implemented")
+
+        assert node_a is not None
+        assert rel is not None
+        assert node_b is not None
+
+        all_props = f", "
+        if props is not None:
+            for k, v in props.items():
+                all_props += f"{k}: '{v}', "
+        all_props = all_props[:-2]
+        query = f"MATCH (a:GraphNode {{GraphID: $graphId, NodeID: $nodeA}}) " \
+                f"MATCH (b:GraphNode {{GraphID: $graphId, NodeID: $nodeB}}) " \
+                f"CALL apoc.create.relationship(a, \"{rel}\", {{ {all_props} }}, b)" \
+                f"YIELD rel RETURN rel"
+        with self.driver.session() as session:
+            session.run(query, graphId=self.graph_id, nodeA=node_a, nodeB=node_b)
 
     def find_matching_nodes(self, *, other_graph) -> Set:
         """
@@ -536,6 +570,7 @@ class Neo4jGraphImporter(ABCGraphImporter):
         # save to file
         with tempfile.NamedTemporaryFile(suffix="-graphml", mode='w') as f1:
             f1.write(graph)
+            f1.flush()
             # read using networkx
             g = nx.read_graphml(f1.name)
 
@@ -661,6 +696,15 @@ class Neo4jGraphImporter(ABCGraphImporter):
                     'call apoc.import.graphml( $fileName, {batchSize: 10000, '
                     'readLabels: true, storeNodeIds: true } ) ',
                     fileName=mapped_file_name).single()
+                # force one common label on all imported nodes
+                self.log.debug(f"Adding GraphNode label to graph {graph_id}")
+                query_string = "MATCH (n {GraphID: $graphId }) SET n:GraphNode"
+                session.run(query_string, graphId=graph_id)
+                # convert class property into a label as well
+                self.log.debug(f"Converting Class property into Neo4j label for all nodes")
+                query_string = "MATCH (n {GraphID: $graphId }) " \
+                               "CALL apoc.create.addLabels([ id(n) ], [ n.Class ]) YIELD node RETURN node"
+                session.run(query_string, graphId=graph_id)
         except Exception as e:
             msg = f"Neo4j APOC import error {str(e)}"
             raise PropertyGraphImportException(graph_id=None, msg=msg)
@@ -699,6 +743,15 @@ class Neo4jGraphImporter(ABCGraphImporter):
                     'call apoc.import.graphml( $fileName, {batchSize: 10000, '
                     'readLabels: true, storeNodeIds: true } ) ',
                     fileName=mapped_file_name).single()
+                # force one common label on all imported nodes
+                self.log.debug(f"Adding GraphNode label to graph {graph_id}")
+                query_string = "MATCH (n {GraphID: $graphId }) SET n:GraphNode"
+                session.run(query_string, graphId=graph_id)
+                # convert class property into a label as well
+                self.log.debug(f"Converting Class property into Neo4j label for all nodes")
+                query_string = "MATCH (n {GraphID: $graphId }) " \
+                               "CALL apoc.create.addLabels([ id(n) ], [ n.Class ]) YIELD node RETURN node"
+                session.run(query_string, graphId=graph_id)
         except Exception as e:
             msg = f"Neo4j APOC import error {str(e)}"
             raise PropertyGraphImportException(graph_id=None, msg=msg)
